// ---------------------------------- Benchmark.h

#ifndef ROOT_TBenchmark
#define ROOT_TBenchmark

//+SEQ,CopyRight,T=NOINCLUDE.


//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TBenchmark                                                           //
//                                                                      //
// This class is a ROOT utility to help benchmarking applications       //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TNamed
//*KEEP,TNAMED.
#include "TNamed.h"
//*KEND.
#endif
#ifndef ROOT_TStopwatch
//*KEEP,TSTOPWATCH.
#include "TStopwatch.h"
//*KEND.
#endif

class TBenchmark : public TNamed {

protected:

   Int_t      fNbench;          //Number of active benchmarks
   Int_t      fNmax;            //Maximum number of benchmarks initialized
   TString    *fNames;          //Names of benchmarks
   Float_t    *fRealTime;       //Real Time
   Float_t    *fCpuTime;        //Cpu Time
   TStopwatch *fTimer;          //Timers

public:
                       TBenchmark();
   virtual            ~TBenchmark();
   Int_t              GetBench(const char *name);
   Float_t            GetCpuTime(const char *name);
   Float_t            GetRealTime(const char *name);
   virtual void       Print(Option_t *name);
   virtual void       Reset();
   virtual void       Show(const char *name);
   virtual void       Start(const char *name);
   virtual void       Stop(const char *name);
   virtual void       Summary();

   ClassDef(TBenchmark,0)  //ROOT utility to help benchmarking applications
};

R__EXTERN TBenchmark  *gBenchmark;

#endif
