
#ifndef ROOT_TControlBar
#define ROOT_TControlBar

//+SEQ,CopyRight,T=NOINCLUDE.

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// TControlBar                                                         //
//                                                                     //
//   ControlBar is fully user configurable tool which provides fast    //
// access to frequently used operations. User can choose between       //
// buttons and drawnbuttons (let's say icons) and assign to them his   //
// own actions (let's say ROOT or C++ commands).                       //
//                                                                     //
/////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TControlBarButton
//*KEEP,TCONTROLBARBUTTON.
#include "TControlBarButton.h"
//*KEND.
#endif

#ifndef ROOT_TControlBarImp
//*KEEP,TCONTROLBARIMP.
#include "TControlBarImp.h"
//*KEND.
#endif

class TList;

class TControlBar : public TControlBarButton {

friend class  TControlBarImp;

private:
   virtual void   Create();
   void           Initialize(Int_t x, Int_t y);

protected:

   TControlBarImp *fControlBarImp;  //system specific implementation
   Int_t           fOrientation;    //orientation
   TList          *fButtons;        //list of buttons
   Int_t           fNoroc;          //number of rows or columns

public:
   enum { kVertical = 1, kHorizontal = 2 };

   TControlBar();
   TControlBar(Text_t *orientation, Text_t *title="");
   TControlBar(Text_t *orientation, Text_t *title, Int_t x, Int_t y);
   virtual ~TControlBar();

   void           AddButton(TControlBarButton *button);
   void           AddButton(TControlBarButton &button);
   void           AddButton(Text_t *label, Text_t *action, Text_t *hint="", Text_t *type="button");
   void           AddControlBar(TControlBar *controlBar);
   void           AddControlBar(TControlBar &controlBar);
   void           AddSeparator();
   TControlBarImp *GetControlBarImp() { return fControlBarImp; }
   TList          *GetListOfButtons() { return fButtons; }
   Int_t          GetNumberOfColumns() { return fNoroc; }
   Int_t          GetNumberOfRows() { return fNoroc; }
   Int_t          GetOrientation() { return fOrientation; }
   void           Hide();
   void           SetNumberOfColumns(Int_t n) { fNoroc = n; }
   void           SetNumberOfRows(Int_t n) { fNoroc = n; }
   void           SetOrientation(Text_t *o);
   void           SetOrientation(Int_t o);
   void           Show();

   ClassDef(TControlBar,0) //Control bar
};

#endif
