
#ifndef ROOT_TEventList
#define ROOT_TEventList

//+SEQ,CopyRight,T=NOINCLUDE.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TEventList                                                           //
//                                                                      //
// A list of selected entries in a TTree.                               //
//                                                                      //
//////////////////////////////////////////////////////////////////////////


#ifndef ROOT_TNamed
//*KEEP,TNamed.
#include "TNamed.h"
//*KEND.
#endif

class TDirectory;

class TEventList : public TNamed {

protected:
        Int_t            fN;           //Number of elements in the list
        Int_t            fSize;        //Size of array
        Int_t            fDelta;       //Increment size
        Int_t            *fList;       //Array of elements
        TDirectory       *fDirectory;  //Pointer to directory holding this tree

public:
        TEventList();
        TEventList(const Text_t *name, const Text_t *title="",Int_t initsize=0, Int_t delta = 0);
        TEventList(const TEventList &list);
        virtual          ~TEventList();
        virtual void     Add(const TEventList *list);
        virtual Bool_t   Contains(Int_t entry);
        virtual void     Enter(Int_t entry);
        TDirectory      *GetDirectory() {return fDirectory;}
        virtual Int_t    GetEntry(Int_t index) const;
        virtual Int_t    GetIndex(Int_t entry) const;
        virtual Int_t   *GetList() const { return fList; }
        virtual Int_t    GetN() const { return fN; }
        virtual Int_t    GetSize() const { return fSize; }
        virtual void     Print(Option_t *option="");
        virtual void     Reset(Option_t *option="");
        virtual void     Resize(Int_t delta=0);
        virtual void     SetDirectory(TDirectory *dir);
        virtual void     Sort();
        virtual void     Subtract(const TEventList *list);

        TEventList& operator=(const TEventList &list);
 friend TEventList  operator+(const TEventList &list1, const TEventList &list2);
 friend TEventList  operator-(const TEventList &list1, const TEventList &list2);

        ClassDef(TEventList,1)  //A list of selected entries in a TTree.
};

#endif

