
#ifndef ROOT_TFunction
#define ROOT_TFunction

//+SEQ,CopyRight,T=NOINCLUDE.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TFunction                                                            //
//                                                                      //
// Dictionary of global functions (global functions are obtained from   //
// CINT).                                                               //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TDictionary
//*KEEP,TDICTIONARY.
#include "TDictionary.h"
//*KEND.
#endif
#ifndef ROOT_TString
//*KEEP,TSTRING.
#include "TString.h"
//*KEND.
#endif

class G__MethodInfo;


class TFunction : public TDictionary {

friend class TCint;

protected:
   G__MethodInfo  *fInfo;            //pointer to CINT function info
   TString         fSignature;       //string containing function signature
   TList          *fMethodArgs;      //list of function arguments

   virtual void    CreateSignature();

public:
   TFunction(G__MethodInfo *info = 0);
   virtual     ~TFunction();
   const char  *GetName() const;
   const char  *GetSignature() const { return fSignature.Data(); }
   const char  *GetTitle() const;
   const char  *GetReturnTypeName() const;
   TList       *GetListOfMethodArgs();
   Int_t        GetNargs() const;
   Int_t        GetNargsOpt() const;
   Int_t        Compare(TObject *obj);
   ULong_t      Hash();
   void        *InterfaceMethod() const;
   Long_t       Property() const;

   ClassDef(TFunction,0)  //Dictionary for global function
};

#endif
