
#ifndef ROOT_TGLabel
#define ROOT_TGLabel

//+SEQ,CopyRight,T=NOINCLUDE.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TGLabel                                                              //
//                                                                      //
// This class handles GUI labels.                                       //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TGFrame
//*KEEP,TGFRAME.
#include "TGFrame.h"
//*KEND.
#endif
#ifndef ROOT_TGDimension
//*KEEP,TGDIMENSION.
#include "TGDimension.h"
//*KEND.
#endif
#ifndef ROOT_TGString
//*KEEP,TGSTRING.
#include "TGString.h"
//*KEND.
#endif



class TGLabel : public TGFrame {

friend class TGClient;

protected:
   TGString      *fText;         // label text
   UInt_t         fTWidth;       // text width
   UInt_t         fTHeight;      // text height
   Int_t          fTMode;        // text drawing mode (ETextJustification)
   Bool_t         fTextChanged;  // has text changed
   GContext_t     fNormGC;       // graphics context used for drawing label
   FontStruct_t   fFontStruct;   // font to draw label

   static GContext_t     fgDefaultGC;
   static FontStruct_t   fgDefaultFontStruct;

   virtual void DoRedraw();

public:
   TGLabel(const TGWindow *p, TGString *text,
           GContext_t norm = fgDefaultGC,
           FontStruct_t font = fgDefaultFontStruct,
           UInt_t options = kChildFrame,
           ULong_t back = fgDefaultFrameBackground);
   TGLabel(const TGWindow *p, const char *text,
           GContext_t norm = fgDefaultGC,
           FontStruct_t font = fgDefaultFontStruct,
           UInt_t options = kChildFrame,
           ULong_t back = fgDefaultFrameBackground);
   virtual ~TGLabel();

   virtual TGDimension GetDefaultSize() const { return TGDimension(fTWidth, fTHeight+1); }
   const TGString *GetText() const { return fText; }
   void SetText(TGString *new_text);
   void SetTextJustify(Int_t tmode) { fTMode = tmode; }

   ClassDef(TGLabel,0)  // A label GUI element
};

#endif
