
#ifndef ROOT_TGTRA
#define ROOT_TGTRA

//+SEQ,CopyRight,T=NOINCLUDE.

////////////////////////////////////////////////////////////////////////////
//                                                                        //
// TGTRA                                                                  //
//                                                                        //
// GTRA is general twisted trapezoid. Essentially this is a TRAP shape,   //
// except this it is twisted in the x, y plane as a function z.           //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TBRIK
//*KEEP,TBRIK.
#include "TBRIK.h"
//*KEND.
#endif

class TGTRA : public TBRIK {

    protected:
        Float_t fTwist;     // twisting parameter
        Float_t fH1;        // half length in y at low z
        Float_t fBl1;       // half length in x at low z and y low edge
        Float_t fTl1;       // half length in x at low z and y high edge
        Float_t fAlpha1;    // angle w.r.t. the y axis
        Float_t fH2;        // half length in y at high z
        Float_t fBl2;       // half length in x at high z and y low edge
        Float_t fTl2;       // half length in x at high z and y high edge
        Float_t fAlpha2;    // angle w.r.t. the y axis

    public:
        TGTRA();
        TGTRA(Text_t *name, Text_t *title, Text_t *material, Float_t dz, Float_t theta, Float_t phi, Float_t twist, Float_t h1,
              Float_t bl1, Float_t tl1, Float_t alpha1, Float_t h2, Float_t bl2, Float_t tl2,
              Float_t alpha2);
        virtual ~TGTRA();

        Float_t         GetTwist()  {return fTwist;}
        Float_t         GetH1()     {return fH1;}
        Float_t         GetBl1()    {return fBl1;}
        Float_t         GetTl1()    {return fTl1;}
        Float_t         GetAlpha1() {return fAlpha1;}
        Float_t         GetH2()     {return fH2;}
        Float_t         GetBl2()    {return fBl2;}
        Float_t         GetTl2()    {return fTl2;}
        Float_t         GetAlpha2() {return fAlpha2;}

        virtual void    SetPoints(Float_t *buff);

        ClassDef(TGTRA,1)  //GTRA shape
};

#endif
