
#ifndef ROOT_TMethodArg
#define ROOT_TMethodArg

//+SEQ,CopyRight,T=NOINCLUDE.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TMethodArg                                                           //
//                                                                      //
// Dictionary interface for a method argument.                          //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TDictionary
//*KEEP,TDICTIONARY.
#include "TDictionary.h"
//*KEND.
#endif

//*KEEP,TDATAMEMBER.
#include "TDataMember.h"
//*KEND.

class TFunction;
class TMethod;
class G__MethodArgInfo;


class TMethodArg : public TDictionary {

private:
   G__MethodArgInfo  *fInfo;         //pointer to CINT method argument info
   TFunction         *fMethod;       //pointer to the method or global function
   TDataMember       *fDataMember;   //TDataMember pointed by this arg,to get values and options from.
   public:
   TMethodArg(G__MethodArgInfo *info = 0, TFunction *method = 0);
   virtual       ~TMethodArg();
   const char    *GetDefault() const;
   TFunction     *GetMethod() const { return fMethod; }
   const char    *GetName() const;
   const char    *GetTitle() const;
   const char    *GetTypeName() const;
   const char    *GetFullTypeName() const;
   Int_t          Compare(TObject *obj);
   ULong_t        Hash();
   Long_t         Property() const;

   TDataMember   *GetDataMember();
   TList         *GetOptions();

   friend class TMethod;

   ClassDef(TMethodArg,0)  //Dictionary for a method argument
};

#endif

