
#ifndef ROOT_TShape
#define ROOT_TShape

//+SEQ,CopyRight,T=NOINCLUDE.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TShape                                                               //
//                                                                      //
// Basic shape class                                                    //
//                                                                      //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TNamed
//*KEEP,TNAMED.
#include "TNamed.h"
//*KEND.
#endif

#ifndef ROOT_TMaterial
//*KEEP,TMATERIAL.
#include "TMaterial.h"
//*KEND.
#endif

#ifndef ROOT_TAttLine
//*KEEP,TATTLINE.
#include "TAttLine.h"
//*KEND.
#endif

#ifndef ROOT_TAttFill
//*KEEP,TATTFILL.
#include "TAttFill.h"
//*KEND.
#endif

#ifndef ROOT_X3DBuffer
//*KEEP,X3DBUFFER.
#include "X3DBuffer.h"
//*KEND.
#endif

#ifndef ROOT_TPolyLine3D
//*KEEP,TPOLYLINE3D.
#include "TPolyLine3D.h"
//*KEND.
#endif

class TNode;


extern "C" { void FillX3DBuffer (X3DBuffer *buff); }

class TShape  : public TNamed, public TAttLine, public TAttFill {

protected:
   Int_t           fNumber;      //Shape number
   Int_t           fVisibility;  //Visibility flag
   TMaterial       *fMaterial;   //Pointer to material
   Int_t           ShapeDistancetoPrimitive(Int_t numPoints, Int_t px, Int_t py);
public:
                   TShape();
                   TShape(Text_t *name, Text_t *title, Text_t *material);
   virtual         ~TShape();
   virtual Bool_t  Is3D () {return kTRUE;}
   TMaterial       *GetMaterial() {return fMaterial;}
   virtual Int_t   GetNumber() {return fNumber;}
           Int_t   GetVisibility() {return fVisibility;}
   virtual void    Paint(Option_t *option="");
   virtual void    PaintGLPoints(Float_t *vertex);
   virtual void    PaintShape(X3DBuffer *buff, Bool_t rangeView=kFALSE);
   virtual void    SetName(const Text_t *name);
   virtual void    SetPoints(Float_t *buffer);
   virtual void    SetVisibility(Int_t vis) {fVisibility = vis;} // *MENU*

   ClassDef(TShape,1)  //Basic shape
};

R__EXTERN TNode *gNode;
R__EXTERN Size3D gSize3D;

inline void TShape::PaintGLPoints(Float_t *) { }
inline void TShape::SetName(const Text_t *) { }

#endif

