
#ifndef ROOT_TWin32InspectImp
#define ROOT_TWin32InspectImp

///////////////////////////////////////////////////////////////
//                                                           //
//   TWin32InspectImp is a special WIN32 type of browser     //
//   to back TObject::Inspect  member function               //
//                                                           //
///////////////////////////////////////////////////////////////

// +SEQ,TInspectImp.
//*KEEP,TWin32ListViewCtrl,T=C++.
#include "TWin32ListViewCtrl.h"
//*KEND.

// +SEQ,TGWin32WindowsObject.
//*KEEP,TWin32BrowserImp,T=C++.
#include "TWin32BrowserImp.h"
//*KEND.

// class TWin32InspectImp : public TInspectImp, public TGWin32WindowsObject, protected TWin32HookViaThread {
// class TWin32InspectImp : public TGWin32WindowsObject {
 class TWin32InspectImp : public TWin32BrowserImp {

private:
  TObject        *fObject;             // Pointer to displayed object
  TObjArray       fWin32CommCtrls;     // Array of the TWin32CommCtrl objects
  Int_t           fCreated;            // flag whether the object was created

  HIMAGELIST      fhSmallIconList;     // List of the small icons
  HIMAGELIST      fhNormalIconList;    // List of the normal icons

  void    CreateIcons(); // Create the list of the general icons

protected:

  virtual void AddValues();


  virtual void CreateInspector(TObject *obj);
  virtual void MakeHeaders();
  virtual void MakeMenu();
  virtual void MakeTitle();
  virtual void MakeStatusBar();
  virtual void MakeToolBar();
  virtual void RecursiveRemove(TObject *obj);



public:
  TWin32InspectImp();  // default ctor
  TWin32InspectImp(TObject *obj, const char *title, UInt_t width=400, UInt_t height=300); // Create an inspector view for TObject
  TWin32InspectImp(TObject *obj, const char *title, Int_t x, Int_t y, UInt_t width=400, UInt_t height=300);
  virtual ~TWin32InspectImp();

  virtual HWND            GetCtrlHandle(Int_t indx=kID_LISTVIEW){ return GetCommCtrl(indx)->GetWindow(); } // Return a WIN32 Window handle for selected control
  virtual TWin32CommCtrl *GetCommCtrl(Int_t indx=kID_LISTVIEW){ return (TWin32CommCtrl *) fWin32CommCtrls[(Int_t) indx]; } // Return a pointer to the selected TWin32CommCtrl class

  virtual void Shift(UInt_t lParam);
  virtual void Show();


  LRESULT APIENTRY OnClose  (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
  LRESULT APIENTRY OnCommand(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
  LRESULT APIENTRY OnCreate (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
  LRESULT APIENTRY OnPaint  (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

  LRESULT APIENTRY OnMouseButton(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
  LRESULT APIENTRY OnNotify     (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
  LRESULT APIENTRY OnSize       (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
  LRESULT APIENTRY OnSysCommand (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

  Bool_t OnSizeCtrls(UINT uMsg,LONG x, LONG y);

//*-*   CallBack functions

  virtual void NewCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void OpenCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void SaveCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void SaveAsCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void PrintCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void CloseCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void CutCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void CopyCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void PasteCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void SelectAllCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void InvertSelectionCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void ToolBarCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void StatusBarCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void LargeIconsCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void SmallIconsCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void DetailsCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
  virtual void RefreshCB(TWin32InspectImp *obj, TVirtualMenuItem *item);
};


#endif
